@@ -14,6 +14,8 @@
 
 #define NR_CANDIDATES 10
 
+extern int uim_fd;
+
 struct preedit_segment {
   int attr;
   char *str;
@@ -109,10 +111,10 @@ clear_cb(void *ptr)
 static void
 pushback_cb(void *ptr, int attr, char *str)
 {
+  IMUIMContext *uic = ptr;
+
   if(!str)
     return;
-
-  IMUIMContext *uic = ptr;
   uic->pseg = realloc(uic->pseg,
 		      sizeof(struct preedit_segment) *
 		     (uic->nr_psegs + 1));
@@ -329,6 +331,7 @@ void focus_in(GtkIMContext *ic)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
   IMUIMContext *cc;
+
   for (cc = context_list.next; cc != &context_list; cc = cc->next) {
     if (cc != uic) {
       if (uic->stat_win) {
@@ -339,6 +342,9 @@ void focus_in(GtkIMContext *ic)
       }
     }
   }
+
+  uim_helper_client_focus_in(uic->uc);
+
   update_status_window(uic);
   if (uic->stat_win) {
     gtk_widget_show(uic->stat_win);
@@ -351,6 +357,9 @@ void focus_out(GtkIMContext *ic)
   if (uic->win && uic->stat_win) {
     gtk_widget_hide(uic->stat_win);
   }
+
+  uim_helper_client_focus_out(uic->uc);
+
   if(uic->cwin.top_win){
     gtk_widget_hide(uic->cwin.top_win);
   }
@@ -362,7 +371,7 @@ void im_uim_reset(GtkIMContext *ic)
 
   uim_reset_context(uic->uc);
 
-  g_warning (gettext("reset called: implement me"));
+  g_warning (_("reset called: implement me"));
 
 }
 
@@ -517,7 +526,7 @@ select_mode(GtkWidget *w, gpointer p)
   int nth = gtk_option_menu_get_history(GTK_OPTION_MENU(is->menu));
   uim_set_mode(is->uc, nth);
   return 0;
-}
+  }
 
 static void
 update_status_menu(IMUIMContext *uic)
@@ -534,7 +543,7 @@ update_status_menu(IMUIMContext *uic)
   }
 
   /* menu */
-  uic->menu = gtk_option_menu_new();
+    uic->menu = gtk_option_menu_new();
   uic->stat_win = gtk_window_new(GTK_WINDOW_POPUP);
   menu = gtk_menu_new();
   for (i = 0; i < nr; i++) {
@@ -547,9 +556,9 @@ update_status_menu(IMUIMContext *uic)
 		       GTK_SIGNAL_FUNC(select_mode), uic);
     gtk_widget_show(item);
   }
-  /**/
+
   hbox = gtk_hbox_new(FALSE, 0);
-  /**/
+
   gtk_container_add(GTK_CONTAINER(uic->stat_win), hbox);
   gtk_option_menu_set_menu(GTK_OPTION_MENU(uic->menu), menu);
   gtk_box_pack_start(GTK_BOX(hbox), uic->menu, FALSE, FALSE, 2);
@@ -853,6 +862,11 @@ im_module_list(GtkIMContextInfo ***contexts,
 void
 im_module_init(GTypeModule *type_module)
 {
+  char *path = uim_helper_get_pathname();
+  uim_fd     = uim_helper_init_client_fd(path);
+  
+  free(path);
+
   uim_init();
   context_list.next = (IMUIMContext *)&context_list;
   context_list.prev = (IMUIMContext *)&context_list;
