@@ -684,6 +684,18 @@
       (set-car! (nthcdr seg (anthy-context-index-list ac)) idx)
       (anthy-update-preedit ac id))))
 
+(define anthy-prop-list
+  '(
+    ("branch" "Ͼ")
+    ("" "Ҥ餬" "Ҥ餬ʤϤǤޤ" "anthy_hira")
+    ("" "" "" "anthy_kana")
+    ("" "ѱѿ" "ѱѿ⡼" "anthy_zenkaku")
+    ("a" "ľ" "ե٥åȤϤǤޤ" "anthy_direct")    
+    ("branch" "ϥ⡼")
+    ("" "޻" "޻" "anthy_roma")
+    ("" "" "" "anthy_kana")
+    ))
+    
 (register-im
  'anthy
  "ja"
