@@ -541,9 +541,16 @@
 	   #t)
        (if (skk-cancel-key key key-state)
 	   (begin
+	     ;; back to kanji mode
 	     (if (skk-context-candidate-window sc)
 		 (im-end-candidate id))
-	     (skk-flush sc)
+	     (skk-context-set-state! sc 'skk-state-kanji)
+	     (skk-context-set-okuri-head! sc "")
+	     (if (car (skk-context-okuri sc))
+		 (skk-context-set-head! sc
+					(cons (car (skk-context-okuri sc))
+					      (skk-context-head sc))))
+	     (skk-context-set-okuri! sc ())
 	     #f)
 	   #t)
        (if (skk-commit-key key key-state)
