@@ -97,15 +97,19 @@ uim_flush_cb(uim_context uc)
 char *
 uim_code_conv(char *str, iconv_t ic)
 {
+  int len;
+  int buflen;
+  char *realbuf;
+  char *outbuf;
+  char *inbuf;
 
   if(!str)
     return NULL;
-
-  int len = strlen(str);
-  int buflen = len * 6+3;
-  char *realbuf = alloca(buflen);
-  char *outbuf = realbuf;
-  char *inbuf = str;
+  len = strlen(str);
+  buflen = (len * 6)+3;
+  realbuf = alloca(buflen);
+  outbuf = realbuf;
+  inbuf = str;
   if (!ic) {
     return strdup(str);
   }
