@@ -7,15 +7,21 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-
+#include <sys/stat.h>
 
 #define MAX_CLIENT 32
 #define BUFFER_SIZE 1024
 
+static int context_id = 0;
+static int clientfd[MAX_CLIENT];
+static int nclients = 0;
+static int focused_context_id;
+
 /*
-  ФΥե뵭һ(file descripter)롣
-  δؿȤϥͥåȥطΥץ񤯺ݤˤ
-  ǥŪˤ褯ȤΤǡؿν餤ˤܤ̤٤
+  prepare file descriptor.
+  1.create socket
+  2.bind socket & file descriptor
+  3.listen file descriptor
 */
 int init_serv_fd(char *path)
 {
@@ -29,12 +35,6 @@ int init_serv_fd(char *path)
 	return -1;
     }
 
-    /*    foo = 1;
-    foo = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &foo, sizeof(&foo));
-    if (foo <0 ) {
-        perror("failed to set reuse option");
-    }
-    */
     myhost.sun_family = PF_UNIX;
     strcpy(myhost.sun_path, path);
 
@@ -44,49 +44,105 @@ int init_serv_fd(char *path)
 	return -1;
     }
 
-    /*SOMAXCONNʤ5򤤤뤳Ȥ¿*/
+    chmod(path, S_IRUSR|S_IWUSR);
+
+    /*Most case, 5 is used instead of SOMAXCONN*/
     foo = listen(fd, SOMAXCONN);
     if ( foo == -1){
 	perror("failed in listen()");
 	return -1;
     }
-    printf("listen%d\n",SOMAXCONN);
+    printf("listen at %s\n",path);
     return fd;
 }
 
+
+int uim_helper_server_new_context_id(void)
+{
+  context_id++;
+  return context_id;
+}
+
 int proc_func(int fd)
 {
-  int recieve_len,i,res;
+  int i,rc;
   char buf[BUFFER_SIZE];
+
   char *content = NULL;
-  FILE *rfile;
+  FILE *rfile, *wfile;
+
 
   rfile = fdopen(fd, "r");
+  wfile = fdopen(fd, "w");
+
+
+  /* use both fd and rfile is safe? I'm not sure...*/
+  while((i = uim_helpler_fd_readable(fd)) > 0){
+    //    fgets(buf, BUFFER_SIZE, rfile );
+    printf("fd readable %d\n",fd);
+    rc = read(fd,buf, sizeof(buf)-1);
+
+    if(rc == 0)
+      return -1;
 
-  fgets(buf, sizeof(buf), rfile );
-    
     if(!content)
-      content = (char*)malloc(strlen(buf));
+      content = (char*)malloc(strlen(buf)+1); 
+    else
+      content = (char*)realloc(content, strlen(content) + strlen(buf)+1);
+    /*strlen not include last '\0'*/
 
-  /*    else
-	content = (char*)realloc(content, strlen(content) + strlen(buf));*/
+    strcat(content, buf);
+  }
+  
+  printf("recieve:%s\n", content);
 
-    strcat(content,buf);
-    printf("%s",content);
- 
-
-  printf("recieve:%s\n",content);
+  if( content ) {
+    parse_content(content, fd);
+  }
+    
 
   free(content);
 
-  return recieve_len;
+  return 1;
 }
 
-process_connection(int serv_fd, int (*proc_func)(int fd) )
+static parse_content(char *content, int fd)
+{
+  char buf[BUFFER_SIZE];
+  int i;
+  if( strncmp(content,"new_context", 11) == 0 ) {
+    
+    printf("new context creating\n");
+    i = snprintf(buf,BUFFER_SIZE,"%d",
+		 uim_helper_server_new_context_id());
+    printf("new context created %s\n",buf);
+  }
+  /*  else if() {
+
+  }
+  else if() {
+
+  }
+  else if() {
+
+  }
+  else if() {
+
+  }*/
+  else {
+
+    strcpy(buf,"OK\n");
+
+  }
+  
+  write(buf,strlen(buf)+1);
+
+}
+
+uim_helper_server_process_connection(int serv_fd, int (*proc_func)(int fd) )
 {
   fd_set readfds;
-  int clientfd[MAX_CLIENT];
-  int nclients = 0;
+
   int i;
   int fd_biggest = 0;
 
@@ -99,7 +155,6 @@ process_connection(int serv_fd, int (*proc_func)(int fd) )
     FD_SET(serv_fd, &readfds);  
     for (i = 0; i < nclients; i ++) {
       FD_SET(clientfd[i], &readfds);
-      printf("fd_set %d\n",clientfd[i]);
     }
     
     if ( select(fd_biggest+1, &readfds, NULL, NULL, NULL) < 0 ) {
@@ -111,13 +166,22 @@ process_connection(int serv_fd, int (*proc_func)(int fd) )
       struct sockaddr_un clientsoc;
       socklen_t len = sizeof(clientsoc);
       int new_fd;
+      FILE *file;
       new_fd = accept(serv_fd, (struct sockaddr *)&clientsoc, &len);
+
+      /*Insert error handleng when nclients > MAX_CLIENT !!!*/
+
       if(new_fd < 0) {
 	perror("accpet failed");
 	continue;
       }
       if(fd_biggest < new_fd )
 	fd_biggest = new_fd;
+
+      file = fdopen(new_fd,"w");
+
+      fputs("OK", file);
+      fputc('\0', file);
 	
       clientfd[nclients] = new_fd;
       nclients++;
@@ -146,15 +210,18 @@ int
 main(int argc, char **argv)
 {
   char *path = uim_helper_get_pathname();
+  unlink(path);
   int serv_fd = init_serv_fd(path);
 
   if (serv_fd < 0) {
-    printf("Failed to init unix domain socket\n");
+    //    printf("Failed to init unix domain socket\n");
     return 0;
   }
   printf("Waiting for connection at %s\n", path);
 
-  process_connection(serv_fd, proc_func);
+  free(path);
+
+  uim_helper_server_process_connection(serv_fd, proc_func);
 
   return 0;
 }
