@@ -24,6 +24,7 @@ static uim_context context_array[CONTEXT_ARRAY_SIZE];
 struct uim_im *uim_im_array;
 int uim_nr_im;
 static int uim_initialized;
+int uim_fd = -1;
 
 void
 uim_set_preedit_cb(uim_context uc,
@@ -38,23 +39,56 @@ uim_set_preedit_cb(uim_context uc,
   uc->preedit_update_cb = update_cb;
 }
 
-static void
-get_context_id(uim_context is)
+int
+uim_get_context_id(uim_context is)
 {
   int i;
+
+  for (i = 0; i < CONTEXT_ARRAY_SIZE; i++) {
+    if (context_array[i] == is) {
+      return is->id;
+    }
+    return NULL;
+  }
+}
+
+
+int
+uim_new_context_id(uim_context is)
+{
+  int i;
+
+  is->id = NULL;
+
   for (i = 0; i < CONTEXT_ARRAY_SIZE; i++) {
     if (!context_array[i]) {
       context_array[i] = is;
-      is->id = i;
-      return;
+
+      sscanf(uim_helper_send_message(uim_fd, "new_context"),
+	     "%d", &(is->id));      
+      break;	
     }
+    
   }
+  if(!is->id)
+    is->id = i + 1;
+  
+  //  printf("new context id:%d\n",(is->id));
+  
+  return is->id; 
 }
 
+
 static void
 put_context_id(uim_context is)
 {
-  context_array[is->id] = 0;
+  int i;
+  for (i = 0; i < CONTEXT_ARRAY_SIZE; i++) {
+    if(context_array[i] && context_array[i]->id == is->id) {
+      context_array[i] = 0;
+      return;
+    }
+  }
 }
 
 uim_context
@@ -72,7 +106,7 @@ uim_create_context(void *ptr,
   if (!uc) {
     return NULL;
   }
-  get_context_id(uc);
+  uim_new_context_id(uc);
   uc->ptr = ptr;
   uc->commit_cb = commit_cb;
   if (!enc) {
@@ -144,7 +178,13 @@ uim_release_context(uim_context uc)
 uim_context
 uim_find_context(int id)
 {
-  return context_array[id];
+  int i;
+  for (i = 0; i < CONTEXT_ARRAY_SIZE; i++) {
+    if (context_array[i]->id == id) {
+      return  context_array[i];
+    }
+  }
+  return NULL;
 }
 
 int
